@@ -114,7 +114,10 @@ typedef struct
 	_ccid_descriptor ccid;
 
 	/* libusb transfer for the polling (or NULL) */
-	_Atomic (struct libusb_transfer *) polling_transfer;
+	pthread_mutex_t polling_transfer_mutex;
+	struct libusb_transfer *polling_transfer;
+	/* whether the polling should be terminated */
+	bool terminate_requested;
 
 	/* pointer to the multislot extension (if any) */
 	struct usbDevice_MultiSlot_Extension *multislot_extension;
@@ -481,6 +484,11 @@ again_libusb:
 					case HID_OMNIKEY_5422:
 					case ALCOR_LINK_AK9567:
 					case ALCOR_LINK_AK9572:
+					case ACS_WALLETMATE:
+					case ACS_ACR1251:
+					case ACS_ACR1252:
+					case ACS_ACR1252IMP:
+					case ACS_ACR1552:
 						max_interface_number = 1; /* 2 interfaces */
 						break;
 
@@ -491,6 +499,7 @@ again_libusb:
 					 */
 					case GEMALTOPROXDU:
 					case GEMALTOPROXSU:
+					case ACS_ACR1581:
 						max_interface_number = 2; /* 3 interfaces */
 						break;
 
@@ -707,6 +716,12 @@ again:
 					|| (HID_OMNIKEY_5422 == readerID)
 					|| (ALCOR_LINK_AK9567 == readerID)
 					|| (ALCOR_LINK_AK9572 == readerID)
+					|| (ACS_WALLETMATE == readerID)
+					|| (ACS_ACR1581 == readerID)
+					|| (ACS_ACR1251 == readerID)
+					|| (ACS_ACR1252 == readerID)
+					|| (ACS_ACR1252IMP == readerID)
+					|| (ACS_ACR1552 == readerID)
 					|| (FEITIANR502DUAL == readerID))
 				{
 					/* use the next interface for the next "slot" */
@@ -729,10 +744,12 @@ again:
 				usbDevice[reader_index].interface = interface;
 				usbDevice[reader_index].real_nb_opened_slots = 1;
 				usbDevice[reader_index].nb_opened_slots = &usbDevice[reader_index].real_nb_opened_slots;
-				atomic_init(&usbDevice[reader_index].polling_transfer, NULL);
+				pthread_mutex_init(&usbDevice[reader_index].polling_transfer_mutex, NULL);
+				usbDevice[reader_index].polling_transfer = NULL;
+				usbDevice[reader_index].terminate_requested = false;
 				usbDevice[reader_index].disconnected = false;
 
-				/* CCID common informations */
+				/* CCID common information */
 				usbDevice[reader_index].ccid.real_bSeq = 0;
 				usbDevice[reader_index].ccid.pbSeq = &usbDevice[reader_index].ccid.real_bSeq;
 				usbDevice[reader_index].ccid.readerID =
@@ -1056,7 +1073,7 @@ status_t CloseUSB(unsigned int reader_index)
 	/* one slot closed */
 	(*usbDevice[reader_index].nb_opened_slots)--;
 
-	/* release the allocated ressources for the last slot only */
+	/* release the allocated resources for the last slot only */
 	if (0 == *usbDevice[reader_index].nb_opened_slots)
 	{
 		struct usbDevice_MultiSlot_Extension *msExt;
@@ -1099,6 +1116,8 @@ status_t CloseUSB(unsigned int reader_index)
 			usbDevice[reader_index].multislot_extension = NULL;
 		}
 
+		pthread_mutex_destroy(&usbDevice[reader_index].polling_transfer_mutex);
+
 		if (usbDevice[reader_index].ccid.gemalto_firmware_features)
 			free(usbDevice[reader_index].ccid.gemalto_firmware_features);
 
@@ -1171,6 +1190,11 @@ const unsigned char *get_ccid_device_descriptor(const struct libusb_interface *u
 	uint8_t last_endpoint;
 #endif
 
+	if (0 == usb_interface->num_altsetting) {
+		/* No interface descriptor available. */
+		return NULL;
+	}
+
 	if (54 == usb_interface->altsetting->extra_length)
 		return usb_interface->altsetting->extra;
 
@@ -1285,6 +1309,10 @@ uint8_t get_ccid_usb_device_address(int reader_index)
 	/* if multiple interfaces use the first one with CCID class type */
 	for (i = *num; i < desc->bNumInterfaces; i++)
 	{
+		if (desc->interface[i].num_altsetting == 0) {
+			/* No interface descriptor available. */
+			continue;
+		}
 		/* CCID Class? */
 		if (desc->interface[i].altsetting->bInterfaceClass == 0xb
 #ifdef ALLOW_PROPRIETARY_CLASS
@@ -1326,7 +1354,7 @@ bool ccid_check_firmware(struct libusb_device_descriptor *desc)
 		{
 			if (DriverOptions & DRIVER_OPTION_USE_BOGUS_FIRMWARE)
 			{
-				DEBUG_INFO3("Firmware (%X.%02X) is bogus! but you choosed to use it",
+				DEBUG_INFO3("Firmware (%X.%02X) is bogus! but you chose to use it",
 					desc->bcdDevice >> 8, desc->bcdDevice & 0xFF);
 				return false;
 			}
@@ -1405,7 +1433,7 @@ static unsigned int *get_data_rates(unsigned int reader_index,
 		return NULL;
 	}
 
-	/* convert in correct endianess */
+	/* convert in correct endianness */
 	for (i=0; i<n; i++)
 	{
 		uint_array[i] = dw2i(buffer, i*4);
@@ -1503,7 +1531,18 @@ int InterruptRead(int reader_index, int timeout /* in ms */)
 		return IFD_COMMUNICATION_ERROR;
 	}
 
-	atomic_store(&usbDevice[reader_index].polling_transfer, transfer);
+	pthread_mutex_lock(&usbDevice[reader_index].polling_transfer_mutex);
+	usbDevice[reader_index].polling_transfer = transfer;
+	bool terminate_requested = usbDevice[reader_index].terminate_requested;
+	usbDevice[reader_index].terminate_requested = false;
+	pthread_mutex_unlock(&usbDevice[reader_index].polling_transfer_mutex);
+
+	// The termination might've been requested by the other thread before the
+	// polling_transfer field was written. In that case, we have to cancel the
+	// transfer here as opposed to InterruptStop().
+	if (terminate_requested) {
+		libusb_cancel_transfer(transfer);
+	}
 
 	while (!completed)
 	{
@@ -1526,7 +1565,9 @@ int InterruptRead(int reader_index, int timeout /* in ms */)
 	actual_length = transfer->actual_length;
 	ret = transfer->status;
 
-	atomic_store(&usbDevice[reader_index].polling_transfer, NULL);
+	pthread_mutex_lock(&usbDevice[reader_index].polling_transfer_mutex);
+	usbDevice[reader_index].polling_transfer = NULL;
+	pthread_mutex_unlock(&usbDevice[reader_index].polling_transfer_mutex);
 	libusb_free_transfer(transfer);
 
 	DEBUG_PERIODIC3("after (%d) (%d)", reader_index, ret);
@@ -1560,8 +1601,6 @@ int InterruptRead(int reader_index, int timeout /* in ms */)
  ****************************************************************************/
 void InterruptStop(int reader_index)
 {
-	struct libusb_transfer *transfer;
-
 	/* Multislot reader: redirect to Multi_InterrupStop */
 	if (usbDevice[reader_index].multislot_extension != NULL)
 	{
@@ -1569,16 +1608,21 @@ void InterruptStop(int reader_index)
 		return;
 	}
 
-	transfer = atomic_load(&usbDevice[reader_index].polling_transfer);
-	if (transfer)
+	pthread_mutex_lock(&usbDevice[reader_index].polling_transfer_mutex);
+	if (usbDevice[reader_index].polling_transfer)
 	{
 		int ret;
 
-		ret = libusb_cancel_transfer(transfer);
+		ret = libusb_cancel_transfer(usbDevice[reader_index].polling_transfer);
 		if (ret < 0)
 			DEBUG_CRITICAL2("libusb_cancel_transfer failed: %s",
 				libusb_error_name(ret));
+	} else {
+		// Indicate that the next attempt to start an interrupt transfer shouldn't
+		// be proceeded.
+		usbDevice[reader_index].terminate_requested = true;
 	}
+	pthread_mutex_unlock(&usbDevice[reader_index].polling_transfer_mutex);
 } /* InterruptStop */
 
 
@@ -1629,8 +1673,9 @@ static void *Multi_PollingProc(void *p_ext)
 			break;
 		}
 
-		atomic_store(&usbDevice[msExt->reader_index].polling_transfer,
-			transfer);
+		pthread_mutex_lock(&usbDevice[msExt->reader_index].polling_transfer_mutex);
+		usbDevice[msExt->reader_index].polling_transfer = transfer;
+		pthread_mutex_unlock(&usbDevice[msExt->reader_index].polling_transfer_mutex);
 
 		completed = 0;
 		while (!completed && !msExt->terminated)
@@ -1656,8 +1701,9 @@ static void *Multi_PollingProc(void *p_ext)
 			}
 		}
 
-		atomic_store(&usbDevice[msExt->reader_index].polling_transfer,
-			NULL);
+		pthread_mutex_lock(&usbDevice[msExt->reader_index].polling_transfer_mutex);
+		usbDevice[msExt->reader_index].polling_transfer = NULL;
+		pthread_mutex_unlock(&usbDevice[msExt->reader_index].polling_transfer_mutex);
 
 		if (0 == rv)
 		{
@@ -1781,22 +1827,22 @@ end:
  ****************************************************************************/
 static void Multi_PollingTerminate(struct usbDevice_MultiSlot_Extension *msExt)
 {
-	struct libusb_transfer *transfer;
-
 	if (msExt && !msExt->terminated)
 	{
 		msExt->terminated = true;
 
-		transfer = atomic_load(&usbDevice[msExt->reader_index].polling_transfer);
+		pthread_mutex_lock(&usbDevice[msExt->reader_index].polling_transfer_mutex);
 
-		if (transfer)
+		if (usbDevice[msExt->reader_index].polling_transfer)
 		{
 			int ret;
 
-			ret = libusb_cancel_transfer(transfer);
+			ret = libusb_cancel_transfer(usbDevice[msExt->reader_index].polling_transfer);
 			if (ret < 0)
 				DEBUG_CRITICAL2("libusb_cancel_transfer failed: %d", ret);
 		}
+
+		pthread_mutex_unlock(&usbDevice[msExt->reader_index].polling_transfer_mutex);
 	}
 } /* Multi_PollingTerminate */
 
@@ -1902,7 +1948,7 @@ static void Multi_InterruptStop(int reader_index)
 
 	pthread_mutex_lock(&msExt->mutex);
 
-	/* Broacast an interrupt to wake-up the slot's thread */
+	/* Broadcast an interrupt to wake-up the slot's thread */
 	msExt->buffer[interrupt_byte] |= interrupt_mask;
 	pthread_cond_broadcast(&msExt->condition);
 
