@@ -24,6 +24,8 @@
 # endif
 #include <errno.h>
 #include <unistd.h>
+#include <stdbool.h>
+#include <zlib.h>
 
 #include "defs.h"
 #include "ccid.h"
@@ -32,11 +34,6 @@
  * returned by bNumClockSupported and bNumDataRatesSupported */
 #undef DISPLAY_EXTRA_VALUES
 
-#ifndef TRUE
-#define TRUE 1
-#define FALSE 0
-#endif
-
 #define BLUE "\33[34m"
 #define RED "\33[31m"
 #define BRIGHT_RED "\33[01;31m"
@@ -44,15 +41,20 @@
 #define MAGENTA "\33[35m"
 #define NORMAL "\33[0m"
 
+#define _OUTPUT_FILENAME "output"
+#define OUTPUT_FILENAME     _OUTPUT_FILENAME ".txt"
+#define OUTPUT_FILENAME_BIN _OUTPUT_FILENAME ".bin"
+
 /* global variables used in ccid_usb.c but defined in ifdhandler.c */
 int LogLevel = 1+2+4+8; /* full debug */
 int DriverOptions = 0;
 
-static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
+static bool ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	struct libusb_device_descriptor desc,
 	struct libusb_config_descriptor *config_desc,
 	int num,
-	const struct libusb_interface *usb_interface);
+	const struct libusb_interface *usb_interface,
+	FILE * fd);
 
 
 /*****************************************************************************
@@ -65,11 +67,13 @@ int main(int argc, char *argv[])
 	libusb_device **devs, *dev;
 	int nb = 0, r, i;
 	unsigned char buffer[256];
-	char class_ff = FALSE;
+	bool class_ff = false;
 	ssize_t cnt;
+	FILE * fd;
+	gzFile zfd;
 
 	if ((argc > 1) && (0 == strcmp(argv[1], "-p")))
-		class_ff = TRUE;
+		class_ff = true;
 
 	r = libusb_init(NULL);
 	if (r < 0)
@@ -86,6 +90,13 @@ int main(int argc, char *argv[])
 		return (int)cnt;
 	}
 
+	fd = fopen(OUTPUT_FILENAME, "w+");
+	if (NULL == fd)
+	{
+		perror("fopen " OUTPUT_FILENAME);
+		return -1;
+	}
+
 	/* for every device */
 	i = 0;
 	while ((dev = devs[i++]) != NULL)
@@ -134,7 +145,7 @@ int main(int argc, char *argv[])
 		{
 			(void)fprintf(stderr, "  Can't get iManufacturer string: %s\n",
 				libusb_error_name(r));
-			if (getuid())
+			if (r != LIBUSB_ERROR_INVALID_PARAM && getuid())
 			{
 				(void)fprintf(stderr,
 					BRIGHT_RED "Please, restart the command as root\n" NORMAL);
@@ -215,14 +226,14 @@ again:
 					 * reader */
 					continue;
 				else
-					return TRUE;
+					return true;
 			}
 			continue;
 		}
 #endif
 
 		(void)ccid_parse_interface_descriptor(handle, desc, config_desc, num,
-			usb_interface);
+			usb_interface, fd);
 		nb++;
 
 #ifndef __APPLE__
@@ -239,6 +250,34 @@ again:
 
 	libusb_exit(NULL);
 
+	printf("\n");
+	rewind(fd);
+	zfd = gzopen(OUTPUT_FILENAME_BIN, "wb");
+	if (NULL == zfd)
+	{
+		perror("gzopen " OUTPUT_FILENAME_BIN);
+		return -1;
+	}
+	while (!feof(fd))
+	{
+		size_t s;
+		char buff[256];
+
+		s = fread(buff, 1, sizeof buff, fd);
+		if (0 == s && ferror(fd))
+		{
+			perror("fread " OUTPUT_FILENAME);
+			return -1;
+		}
+		fwrite(buff, s, 1, stdout);
+		gzfwrite(buff, s, 1, zfd);
+	}
+	fclose(fd);
+	gzclose(zfd);
+
+	/* remove now useless output.txt */
+	unlink(OUTPUT_FILENAME);
+
 	return 0;
 } /* main */
 
@@ -248,11 +287,12 @@ again:
  *					Parse a CCID USB Descriptor
  *
  ****************************************************************************/
-static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
+static bool ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	struct libusb_device_descriptor desc,
 	struct libusb_config_descriptor *config_desc,
 	int num,
-	const struct libusb_interface *usb_interface)
+	const struct libusb_interface *usb_interface,
+	FILE *fd)
 {
 	const struct libusb_interface_descriptor *usb_interface_descriptor;
 	const unsigned char *device_descriptor;
@@ -262,7 +302,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	/*
 	 * Vendor/model name
 	 */
-	(void)printf(" idVendor: 0x%04X\n", desc.idVendor);
+	(void)fprintf(fd, " idVendor: 0x%04X\n", desc.idVendor);
 	r = libusb_get_string_descriptor_ascii(handle, desc.iManufacturer,
 		buffer, sizeof(buffer));
 	if (r < 0)
@@ -273,147 +313,147 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 		{
 			(void)fprintf(stderr,
 				BRIGHT_RED "Please, restart the command as root\n\n" NORMAL);
-			return TRUE;
+			return true;
 		}
 	}
 	else
-		(void)printf("  iManufacturer: %s\n", buffer);
+		(void)fprintf(fd, "  iManufacturer: %s\n", buffer);
 
-	(void)printf(" idProduct: 0x%04X\n", desc.idProduct);
+	(void)fprintf(fd, " idProduct: 0x%04X\n", desc.idProduct);
 	r =  libusb_get_string_descriptor_ascii(handle, desc.iProduct,
 		buffer, sizeof(buffer));
 	if (r < 0)
 		(void)printf("  Can't get iProduct string: %s\n",
 			libusb_error_name(r));
 	else
-		(void)printf("  iProduct: %s\n", buffer);
+		(void)fprintf(fd, "  iProduct: %s\n", buffer);
 
-	(void)printf(" bcdDevice: %X.%02X (firmware release?)\n",
+	(void)fprintf(fd, " bcdDevice: %X.%02X (firmware release?)\n",
 		desc.bcdDevice >> 8, desc.bcdDevice & 0xFF);
 
 	usb_interface_descriptor = get_ccid_usb_interface(config_desc, &num)->altsetting;
 
-	(void)printf(" bLength: %d\n", usb_interface_descriptor->bLength);
+	(void)fprintf(fd, " bLength: %d\n", usb_interface_descriptor->bLength);
 
-	(void)printf(" bDescriptorType: %d\n", usb_interface_descriptor->bDescriptorType);
+	(void)fprintf(fd, " bDescriptorType: %d\n", usb_interface_descriptor->bDescriptorType);
 
-	(void)printf(" bInterfaceNumber: %d\n", usb_interface_descriptor->bInterfaceNumber);
+	(void)fprintf(fd, " bInterfaceNumber: %d\n", usb_interface_descriptor->bInterfaceNumber);
 
-	(void)printf(" bAlternateSetting: %d\n", usb_interface_descriptor->bAlternateSetting);
+	(void)fprintf(fd, " bAlternateSetting: %d\n", usb_interface_descriptor->bAlternateSetting);
 
-	(void)printf(" bNumEndpoints: %d\n", usb_interface_descriptor->bNumEndpoints);
+	(void)fprintf(fd, " bNumEndpoints: %d\n", usb_interface_descriptor->bNumEndpoints);
 	switch (usb_interface_descriptor->bNumEndpoints)
 	{
 		case 0:
-			(void)printf("  Control only\n");
+			(void)fprintf(fd, "  Control only\n");
 			break;
 		case 1:
-			(void)printf("  Interrupt-IN\n");
+			(void)fprintf(fd, "  Interrupt-IN\n");
 			break;
 		case 2:
-			(void)printf("  bulk-IN and bulk-OUT\n");
+			(void)fprintf(fd, "  bulk-IN and bulk-OUT\n");
 			break;
 		case 3:
-			(void)printf("  bulk-IN, bulk-OUT and Interrupt-IN\n");
+			(void)fprintf(fd, "  bulk-IN, bulk-OUT and Interrupt-IN\n");
 			break;
 		default:
-			(void)printf("  UNKNOWN value\n");
+			(void)fprintf(fd, "  UNKNOWN value\n");
 	}
 
-	(void)printf(" bInterfaceClass: 0x%02X", usb_interface_descriptor->bInterfaceClass);
+	(void)fprintf(fd, " bInterfaceClass: 0x%02X", usb_interface_descriptor->bInterfaceClass);
 	if (usb_interface_descriptor->bInterfaceClass == 0x0b)
-		(void)printf(" [Chip Card Interface Device Class (CCID)]\n");
+		(void)fprintf(fd, " [Chip Card Interface Device Class (CCID)]\n");
 	else
 	{
-		(void)printf("\n  NOT A CCID DEVICE\n");
+		(void)fprintf(fd, "\n  NOT A CCID DEVICE\n");
 		if (usb_interface_descriptor->bInterfaceClass != 0xFF)
-			return TRUE;
+			return true;
 		else
-			(void)printf("  Class is 0xFF (proprietary)\n");
+			(void)fprintf(fd, "  Class is 0xFF (proprietary)\n");
 	}
 
-	(void)printf(" bInterfaceSubClass: %d\n",
+	(void)fprintf(fd, " bInterfaceSubClass: %d\n",
 		usb_interface_descriptor->bInterfaceSubClass);
 	if (usb_interface_descriptor->bInterfaceSubClass)
-		(void)printf("  UNSUPPORTED SubClass\n");
+		(void)fprintf(fd, "  UNSUPPORTED SubClass\n");
 
-	(void)printf(" bInterfaceProtocol: %d\n",
+	(void)fprintf(fd, " bInterfaceProtocol: %d\n",
 		usb_interface_descriptor->bInterfaceProtocol);
 	switch (usb_interface_descriptor->bInterfaceProtocol)
 	{
 		case 0:
-			(void)printf("  bulk transfer, optional interrupt-IN (CCID)\n");
+			(void)fprintf(fd, "  bulk transfer, optional interrupt-IN (CCID)\n");
 			break;
 		case 1:
-			(void)printf("  ICCD Version A, Control transfers, (no interrupt-IN)\n");
+			(void)fprintf(fd, "  ICCD Version A, Control transfers, (no interrupt-IN)\n");
 			break;
 		case 2:
-			(void)printf("  ICCD Version B, Control transfers, (optional interrupt-IN)\n");
+			(void)fprintf(fd, "  ICCD Version B, Control transfers, (optional interrupt-IN)\n");
 			break;
 		default:
-			(void)printf("  UNSUPPORTED InterfaceProtocol\n");
+			(void)fprintf(fd, "  UNSUPPORTED InterfaceProtocol\n");
 	}
 
 	r = libusb_get_string_descriptor_ascii(handle, usb_interface_descriptor->iInterface,
 		buffer, sizeof(buffer));
 	if (r < 0)
-		(void)printf(" Can't get iInterface string: %s\n",
+		(void)fprintf(fd, " Can't get iInterface string: %s\n",
 			libusb_error_name(r));
 	else
-		(void)printf(" iInterface: %s\n", buffer);
+		(void)fprintf(fd, " iInterface: %s\n", buffer);
 
 	device_descriptor = get_ccid_device_descriptor(usb_interface);
 	if (NULL == device_descriptor)
 	{
-		(void)printf("\n  NOT A CCID DEVICE\n");
-		return TRUE;
+		(void)fprintf(fd, "\n  NOT A CCID DEVICE\n");
+		return true;
 	}
 
 	/*
 	 * CCID Class Descriptor
 	 */
-	(void)printf(" CCID Class Descriptor\n");
+	(void)fprintf(fd, " CCID Class Descriptor\n");
 
-	(void)printf("  bLength: 0x%02X\n", device_descriptor[0]);
+	(void)fprintf(fd, "  bLength: 0x%02X\n", device_descriptor[0]);
 	if (device_descriptor[0] != 0x36)
 	{
-		(void)printf("   UNSUPPORTED bLength\n");
-		return TRUE;
+		(void)fprintf(fd, "   UNSUPPORTED bLength\n");
+		return true;
 	}
 
-	(void)printf("  bDescriptorType: 0x%02X\n", device_descriptor[1]);
+	(void)fprintf(fd, "  bDescriptorType: 0x%02X\n", device_descriptor[1]);
 	if (device_descriptor[1] != 0x21)
 	{
 		if (0xFF == device_descriptor[1])
-			(void)printf("   PROPRIETARY bDescriptorType\n");
+			(void)fprintf(fd, "   PROPRIETARY bDescriptorType\n");
 		else
 		{
-			(void)printf("   UNSUPPORTED bDescriptorType\n");
-			return TRUE;
+			(void)fprintf(fd, "   UNSUPPORTED bDescriptorType\n");
+			return true;
 		}
 	}
 
-	(void)printf("  bcdCCID: %X.%02X\n", device_descriptor[3], device_descriptor[2]);
-	(void)printf("  bMaxSlotIndex: 0x%02X\n", device_descriptor[4]);
-	(void)printf("  bVoltageSupport: 0x%02X\n", device_descriptor[5]);
+	(void)fprintf(fd, "  bcdCCID: %X.%02X\n", device_descriptor[3], device_descriptor[2]);
+	(void)fprintf(fd, "  bMaxSlotIndex: 0x%02X\n", device_descriptor[4]);
+	(void)fprintf(fd, "  bVoltageSupport: 0x%02X\n", device_descriptor[5]);
 	if (device_descriptor[5] & 0x01)
-		(void)printf("   5.0V\n");
+		(void)fprintf(fd, "   5.0V\n");
 	if (device_descriptor[5] & 0x02)
-		(void)printf("   3.0V\n");
+		(void)fprintf(fd, "   3.0V\n");
 	if (device_descriptor[5] & 0x04)
-		(void)printf("   1.8V\n");
+		(void)fprintf(fd, "   1.8V\n");
 
-	(void)printf("  dwProtocols: 0x%02X%02X 0x%02X%02X\n", device_descriptor[9], device_descriptor[8],
+	(void)fprintf(fd, "  dwProtocols: 0x%02X%02X 0x%02X%02X\n", device_descriptor[9], device_descriptor[8],
 		device_descriptor[7], device_descriptor[6]);
 	if (device_descriptor[6] & 0x01)
-		(void)printf("   T=0\n");
+		(void)fprintf(fd, "   T=0\n");
 	if (device_descriptor[6] & 0x02)
-		(void)printf("   T=1\n");
+		(void)fprintf(fd, "   T=1\n");
 
-	(void)printf("  dwDefaultClock: %.3f MHz\n", dw2i(device_descriptor, 10)/1000.0);
-	(void)printf("  dwMaximumClock: %.3f MHz\n", dw2i(device_descriptor, 14)/1000.0);
+	(void)fprintf(fd, "  dwDefaultClock: %.3f MHz\n", dw2i(device_descriptor, 10)/1000.0);
+	(void)fprintf(fd, "  dwMaximumClock: %.3f MHz\n", dw2i(device_descriptor, 14)/1000.0);
 	int bNumClockSupported = device_descriptor[18];
-	(void)printf("  bNumClockSupported: %d%s\n", bNumClockSupported,
+	(void)fprintf(fd, "  bNumClockSupported: %d%s\n", bNumClockSupported,
 		bNumClockSupported ? "" : " (will use whatever is returned)");
 	{
 		int n;
@@ -435,17 +475,17 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 		/* we got an error? */
 		if (n <= 0)
 		{
-			(void)(void)printf("   IFD does not support GET CLOCK FREQUENCIES request: %s\n", libusb_error_name(n));
+			(void)(void)fprintf(fd, "   IFD does not support GET CLOCK FREQUENCIES request: %s\n", libusb_error_name(n));
 			if (EBUSY == errno)
 			{
 				(void)fprintf(stderr,
 					BRIGHT_RED "   Please, stop pcscd and retry\n\n" NORMAL);
-				return TRUE;
+				return true;
 			}
 		}
 		else
 			if (n % 4)	/* not a multiple of 4 */
-				(void)printf("   wrong size for GET CLOCK FREQUENCIES: %d\n", n);
+				(void)fprintf(fd, "   wrong size for GET CLOCK FREQUENCIES: %d\n", n);
 			else
 			{
 				int i;
@@ -453,7 +493,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 				/* we do not get the expected number of data rates */
 				if ((n != bNumClockSupported*4) && bNumClockSupported)
 				{
-					(void)printf("   Got %d clock frequencies but was expecting %d\n",
+					(void)fprintf(fd, "   Got %d clock frequencies but was expecting %d\n",
 						n/4, bNumClockSupported);
 
 					/* we got more data than expected */
@@ -464,13 +504,13 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 				}
 
 				for (i=0; i<n; i+=4)
-					(void)printf("   Support %d kHz\n", dw2i(buffer, i));
+					(void)fprintf(fd, "   Support %d kHz\n", dw2i(buffer, i));
 			}
 	}
-	(void)printf("  dwDataRate: %d bps\n", dw2i(device_descriptor, 19));
-	(void)printf("  dwMaxDataRate: %d bps\n", dw2i(device_descriptor, 23));
+	(void)fprintf(fd, "  dwDataRate: %d bps\n", dw2i(device_descriptor, 19));
+	(void)fprintf(fd, "  dwMaxDataRate: %d bps\n", dw2i(device_descriptor, 23));
 	int bNumDataRatesSupported = device_descriptor[27];
-	(void)printf("  bNumDataRatesSupported: %d%s\n", bNumDataRatesSupported,
+	(void)fprintf(fd, "  bNumDataRatesSupported: %d%s\n", bNumDataRatesSupported,
 		bNumDataRatesSupported ? "" : " (will use whatever is returned)");
 	{
 		int n;
@@ -494,11 +534,11 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 
 		/* we got an error? */
 		if (n <= 0)
-			(void)printf("   IFD does not support GET_DATA_RATES request: %s\n",
+			(void)fprintf(fd, "   IFD does not support GET_DATA_RATES request: %s\n",
 				libusb_error_name(n));
 		else
 			if (n % 4)	/* not a multiple of 4 */
-				(void)printf("   wrong size for GET_DATA_RATES: %d\n", n);
+				(void)fprintf(fd, "   wrong size for GET_DATA_RATES: %d\n", n);
 			else
 			{
 				int i;
@@ -506,7 +546,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 				/* we do not get the expected number of data rates */
 				if ((n != bNumDataRatesSupported*4) && bNumDataRatesSupported)
 				{
-					(void)printf("   Got %d data rates but was expecting %d\n", n/4,
+					(void)fprintf(fd, "   Got %d data rates but was expecting %d\n", n/4,
 						bNumDataRatesSupported);
 
 					/* we got more data than expected */
@@ -517,95 +557,95 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 				}
 
 				for (i=0; i<n; i+=4)
-					(void)printf("   Support %d bps\n", dw2i(buffer, i));
+					(void)fprintf(fd, "   Support %d bps\n", dw2i(buffer, i));
 			}
 	}
-	(void)printf("  dwMaxIFSD: %d\n", dw2i(device_descriptor, 28));
-	(void)printf("  dwSynchProtocols: 0x%08X\n", dw2i(device_descriptor, 32));
+	(void)fprintf(fd, "  dwMaxIFSD: %d\n", dw2i(device_descriptor, 28));
+	(void)fprintf(fd, "  dwSynchProtocols: 0x%08X\n", dw2i(device_descriptor, 32));
 	if (device_descriptor[32] & 0x01)
-			(void)printf("   2-wire protocol\n");
+			(void)fprintf(fd, "   2-wire protocol\n");
 	if (device_descriptor[32] & 0x02)
-			(void)printf("   3-wire protocol\n");
+			(void)fprintf(fd, "   3-wire protocol\n");
 	if (device_descriptor[32] & 0x04)
-			(void)printf("   I2C protocol\n");
+			(void)fprintf(fd, "   I2C protocol\n");
 
-	(void)printf("  dwMechanical: 0x%08X\n", dw2i(device_descriptor, 36));
+	(void)fprintf(fd, "  dwMechanical: 0x%08X\n", dw2i(device_descriptor, 36));
 	if (device_descriptor[36] == 0)
-		(void)printf("   No special characteristics\n");
+		(void)fprintf(fd, "   No special characteristics\n");
 	if (device_descriptor[36] & 0x01)
-		(void)printf("   Card accept mechanism\n");
+		(void)fprintf(fd, "   Card accept mechanism\n");
 	if (device_descriptor[36] & 0x02)
-		(void)printf("   Card ejection mechanism\n");
+		(void)fprintf(fd, "   Card ejection mechanism\n");
 	if (device_descriptor[36] & 0x04)
-		(void)printf("   Card capture mechanism\n");
+		(void)fprintf(fd, "   Card capture mechanism\n");
 	if (device_descriptor[36] & 0x08)
-		(void)printf("   Card lock/unlock mechanism\n");
+		(void)fprintf(fd, "   Card lock/unlock mechanism\n");
 
-	(void)printf("  dwFeatures: 0x%08X\n", dw2i(device_descriptor, 40));
+	(void)fprintf(fd, "  dwFeatures: 0x%08X\n", dw2i(device_descriptor, 40));
 	if (dw2i(device_descriptor, 40) == 0)
-		(void)printf("   No special characteristics\n");
+		(void)fprintf(fd, "   No special characteristics\n");
 	if (device_descriptor[40] & 0x02)
-		(void)printf("   ....02 Automatic parameter configuration based on ATR data\n");
+		(void)fprintf(fd, "   ....02 Automatic parameter configuration based on ATR data\n");
 	if (device_descriptor[40] & 0x04)
-		(void)printf("   ....04 Automatic activation of ICC on inserting\n");
+		(void)fprintf(fd, "   ....04 Automatic activation of ICC on inserting\n");
 	if (device_descriptor[40] & 0x08)
-		(void)printf("   ....08 Automatic ICC voltage selection\n");
+		(void)fprintf(fd, "   ....08 Automatic ICC voltage selection\n");
 	if (device_descriptor[40] & 0x10)
-		(void)printf("   ....10 Automatic ICC clock frequency change according to parameters\n");
+		(void)fprintf(fd, "   ....10 Automatic ICC clock frequency change according to parameters\n");
 	if (device_descriptor[40] & 0x20)
-		(void)printf("   ....20 Automatic baud rate change according to frequency and Fi, Di params\n");
+		(void)fprintf(fd, "   ....20 Automatic baud rate change according to frequency and Fi, Di params\n");
 	if (device_descriptor[40] & 0x40)
-		(void)printf("   ....40 Automatic parameters negotiation made by the CCID\n");
+		(void)fprintf(fd, "   ....40 Automatic parameters negotiation made by the CCID\n");
 	if (device_descriptor[40] & 0x80)
-		(void)printf("   ....80 Automatic PPS made by the CCID\n");
+		(void)fprintf(fd, "   ....80 Automatic PPS made by the CCID\n");
 	if (device_descriptor[41] & 0x01)
-		(void)printf("   ..01.. CCID can set ICC in clock stop mode\n");
+		(void)fprintf(fd, "   ..01.. CCID can set ICC in clock stop mode\n");
 	if (device_descriptor[41] & 0x02)
-		(void)printf("   ..02.. NAD value other than 00 accepted (T=1)\n");
+		(void)fprintf(fd, "   ..02.. NAD value other than 00 accepted (T=1)\n");
 	if (device_descriptor[41] & 0x04)
-		(void)printf("   ..04.. Automatic IFSD exchange as first exchange (T=1)\n");
+		(void)fprintf(fd, "   ..04.. Automatic IFSD exchange as first exchange (T=1)\n");
 	if (device_descriptor[41] & 0x08)
-		(void)printf("   ..08.. ICCD token\n");
+		(void)fprintf(fd, "   ..08.. ICCD token\n");
 	switch (device_descriptor[42] & 0x07)
 	{
 		case 0x00:
-			(void)printf("   00.... Character level exchange\n");
+			(void)fprintf(fd, "   00.... Character level exchange\n");
 			break;
 
 		case 0x01:
-			(void)printf("   01.... TPDU level exchange\n");
+			(void)fprintf(fd, "   01.... TPDU level exchange\n");
 			break;
 
 		case 0x02:
-			(void)printf("   02.... Short APDU level exchange\n");
+			(void)fprintf(fd, "   02.... Short APDU level exchange\n");
 			break;
 
 		case 0x04:
-			(void)printf("   04.... Short and Extended APDU level exchange\n");
+			(void)fprintf(fd, "   04.... Short and Extended APDU level exchange\n");
 			break;
 	}
 	if (device_descriptor[42] & 0x10)
-		(void)printf("   10.... USB Wake up signaling supported on card insertion and removal\n");
+		(void)fprintf(fd, "   10.... USB Wake up signaling supported on card insertion and removal\n");
 
-	(void)printf("  dwMaxCCIDMessageLength: %d bytes\n", dw2i(device_descriptor, 44));
-	(void)printf("  bClassGetResponse: 0x%02X\n", device_descriptor[48]);
+	(void)fprintf(fd, "  dwMaxCCIDMessageLength: %d bytes\n", dw2i(device_descriptor, 44));
+	(void)fprintf(fd, "  bClassGetResponse: 0x%02X\n", device_descriptor[48]);
 	if (0xFF == device_descriptor[48])
-		(void)printf("   echoes the APDU class\n");
-	(void)printf("  bClassEnvelope: 0x%02X\n", device_descriptor[49]);
+		(void)fprintf(fd, "   echoes the APDU class\n");
+	(void)fprintf(fd, "  bClassEnvelope: 0x%02X\n", device_descriptor[49]);
 	if (0xFF == device_descriptor[49])
-		(void)printf("   echoes the APDU class\n");
-	(void)printf("  wLcdLayout: 0x%04X\n", (device_descriptor[51] << 8)+device_descriptor[50]);
+		(void)fprintf(fd, "   echoes the APDU class\n");
+	(void)fprintf(fd, "  wLcdLayout: 0x%04X\n", (device_descriptor[51] << 8)+device_descriptor[50]);
 	if (device_descriptor[51])
-		(void)printf("   %2d lines\n", device_descriptor[51]);
+		(void)fprintf(fd, "   %2d lines\n", device_descriptor[51]);
 	if (device_descriptor[50])
-		(void)printf("   %2d characters per line\n", device_descriptor[50]);
-	(void)printf("  bPINSupport: 0x%02X\n", device_descriptor[52]);
+		(void)fprintf(fd, "   %2d characters per line\n", device_descriptor[50]);
+	(void)fprintf(fd, "  bPINSupport: 0x%02X\n", device_descriptor[52]);
 	if (device_descriptor[52] & 0x01)
-		(void)printf("   PIN Verification supported\n");
+		(void)fprintf(fd, "   PIN Verification supported\n");
 	if (device_descriptor[52] & 0x02)
-		(void)printf("   PIN Modification supported\n");
-	(void)printf("  bMaxCCIDBusySlots: %d\n", device_descriptor[53]);
+		(void)fprintf(fd, "   PIN Modification supported\n");
+	(void)fprintf(fd, "  bMaxCCIDBusySlots: %d\n", device_descriptor[53]);
 
-	return FALSE;
+	return false;
 } /* ccid_parse_interface_descriptor */
 
